-- Liefert als initialen Owner die ID des struktur-owner Nutzers.
--
-- Im JMD-Strukturteil sollen alle Akten dem struktur-owner Nutzer
-- gehören.  Daher wird initial_master_ds_owner_id umdefiniert, um
-- dessen ID zurückzugeben, sofern der eingeloggte Nutzer ein Vertreter
-- dieses struktur-owners ist.  Wenn der eingeloggte Nutzer kein
-- Vertreter ist, wird eine Exception erzeugt.
CREATE OR REPLACE FUNCTION initial_master_ds_owner_id() RETURNS integer
    LANGUAGE 'plpgsql' STABLE
    AS $$
DECLARE
    owner_id        integer;
    standingroup_id integer;
    my_id           integer;
BEGIN
    SELECT INTO owner_id, standingroup_id id, vertreter FROM ka_benutzer_tbl
           WHERE login LIKE '%_struktur-owner';
    SELECT INTO my_id id FROM ka_benutzer_tbl WHERE login = session_user;
    IF my_id IS NULL
       OR my_id NOT IN (SELECT benutzer_id FROM nm_benutzer_gruppe_tbl_view
                        WHERE gruppe_id = standingroup_id)
    THEN
        RAISE EXCEPTION
	'User % is not a member of struktur-nutzer standin group',
	session_user;
    ELSE
        RETURN owner_id;
    END IF;
END;
$$;

--
-- Erzeugt den Nutzer struktur-owner und die Gruppe struktur-nutzer.
-- Die struktur-nutzer Gruppe wird als Vertretergruppe des
-- struktur-owner eingetragen.
CREATE OR REPLACE FUNCTION jmd_init_struktur_db (
        agency          varchar
    ) RETURNS VOID
    LANGUAGE 'plpgsql' VOLATILE
    AS $$
DECLARE
    ownergroupid    integer;
    standingroupid  integer;
BEGIN
    -- Standard struktur-user und zugehörige Gruppe
    SELECT INTO ownergroupid create_benutzergruppe_ds('struktur-owner');
    SELECT INTO standingroupid create_benutzergruppe_ds('struktur-nutzer');

    INSERT INTO ka_benutzer_tbl (login, passwort_aendern, gid, vertreter)
    VALUES ('ka_' || agency || '_' || 'struktur-owner', False,
            ownergroupid, standingroupid);
END;
$$ ;


-- View um nur die zum Zugriff auf die Strukturakten berechtigten Nutzer
-- anzuzeigen.
CREATE OR REPLACE VIEW jmd_struktur_nutzer_view
AS SELECT b.id, b.login
   FROM ka_benutzer_tbl b JOIN nm_benutzer_gruppe_tbl g
        ON b.id = g.benutzer_id
   WHERE g.gruppe_id = (SELECT id FROM ka_benutzergruppe_tbl
           	        WHERE name = 'struktur-nutzer');

GRANT SELECT ON jmd_struktur_nutzer_view TO GROUP :cm_group;
GRANT SELECT ON jmd_struktur_nutzer_view TO GROUP :admin_group;


-- Bestimmt, ob der session_user auf die Strukturakten zugreifen darf.
CREATE OR REPLACE FUNCTION jmd_check_struktur_permission ()
    RETURNS boolean
    LANGUAGE 'plpgsql' STABLE
    AS $$
BEGIN
    RETURN EXISTS (SELECT 1 FROM jmd_struktur_nutzer_view 
                   WHERE login = session_user);
END;
$$ ;

-- Löscht alle Mitglieder der struktur-nutzer Gruppe.
--
-- Danach sollte sie mit jmdstruktur_add_struktur_group_user wieder
-- befüllt werden
CREATE OR REPLACE FUNCTION jmd_clear_struktur_nutzer_group ()
    RETURNS VOID
    LANGUAGE 'plpgsql' VOLATILE SECURITY DEFINER
    AS $$
DECLARE
    groupid    integer;
BEGIN
    DELETE FROM ka_benutzer_tbl
    WHERE id IN (SELECT benutzer_id FROM nm_benutzer_gruppe_tbl
                 WHERE gruppe_id = (SELECT id FROM ka_benutzergruppe_tbl
                                    WHERE name = 'struktur-nutzer'));
END;
$$ ;

REVOKE ALL ON FUNCTION jmd_clear_struktur_nutzer_group() FROM PUBLIC;
GRANT EXECUTE ON FUNCTION jmd_clear_struktur_nutzer_group() TO GROUP :admin_group;


-- Fügt ein Mitglied zur struktur-nutzer Gruppe hinzu.
CREATE OR REPLACE FUNCTION jmd_add_struktur_nutzer (
        agency          varchar,
	loginname       varchar,
	firstname       varchar,
	surname         varchar
    ) RETURNS VOID
    LANGUAGE 'plpgsql' VOLATILE SECURITY DEFINER
    AS $$
DECLARE
    full_login varchar;
    userid  integer;
BEGIN
    full_login := 'ka_' || agency || '_' || loginname;

    INSERT INTO ka_benutzer_tbl (login, vorname, nachname, passwort_aendern)
    VALUES (full_login, firstname, surname, False);
    SELECT INTO userid currval('ka_benutzer_tbl_id_seq');

    INSERT INTO nm_benutzer_gruppe_tbl (gruppe_id, benutzer_id)
    VALUES ((SELECT vertreter FROM ka_benutzer_tbl
             WHERE login = 'ka_' || agency || '_' || 'struktur-owner'),
	    userid);
END;
$$ ;

REVOKE ALL ON FUNCTION jmd_add_struktur_nutzer(varchar, varchar, varchar, varchar) FROM PUBLIC;
GRANT EXECUTE ON FUNCTION jmd_add_struktur_nutzer(varchar, varchar, varchar, varchar) TO GROUP :admin_group;


-- Remove a user from the ka_benutzer_tbl
CREATE OR REPLACE FUNCTION jmd_delete_struktur_nutzer (
    user_id       integer
    ) RETURNS VOID
    LANGUAGE 'plpgsql' VOLATILE SECURITY DEFINER
    AS $$
BEGIN
    DELETE FROM ka_benutzer_tbl WHERE id = user_id;
END;
$$ ;

REVOKE ALL ON FUNCTION jmd_delete_struktur_nutzer(integer) FROM PUBLIC;
GRANT EXECUTE ON FUNCTION jmd_delete_struktur_nutzer(integer) TO GROUP :admin_group;
