# -*- coding: utf-8 -*-
#
# Copyright 2007, 2008, 2010, 2020 Intevation GmbH, Germany,
# <info@intevation.de>
#
# This file is part of mpuls WASKA (CoMPUter-based case fiLeS - 
# Web-Anwendungs-Server fuer Kompetenzagenturen).
# 
# mpuls WASKA is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the
# Free Software Foundation, either version 3 of the License, or (at your
# option) any later version.
# 
# mpuls WASKA is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public
# License for more details.
# 
# You should have received a copy of the GNU Affero General Public
# License along with mpuls WASKA. If not, see <http://www.gnu.org/licenses/>.
# 
# mpuls WASKA has been developed on behalf of the 
# Projekttraeger im Deutschen Zentrum fuer Luft- und Raumfahrt e.V. (PT-DLR)
# within the programme Kompetenzagenturen (Durchfuehrungsphase) funded by
# the Bundesministerium fuer Familie, Senioren, Frauen und Jugend and 
# European Social Fund resources.
#
# Authors:
# Sascha L. Teichmann <teichmann@intevation.de>
# Magnus Schieder <magnus.schieder@intevation.de>
#

from cgi import escape
import logging

# nl2br is re-exported from this module for convenient use in templates.
# It works with unicode objects as well as MarkupSafe/literal objects
# and automatically escapes text in unicode objects. It always returns
# an instance of literal.
from webhelpers.html.converters import nl2br



from HTMLParser import HTMLParser
from webhelpers.html import literal
from htmlentitydefs import name2codepoint


log = logging.getLogger(__name__)


class _DeHTMLParser(HTMLParser):
    def __init__(self):
        HTMLParser.__init__(self)
        self._paragraphs = []
        self._current_paragraph = None
        self._current_line = None

    def _append_chardata(self, data):
        if self._current_paragraph is None:
            self._current_paragraph = []
        if self._current_line is None:
            self._current_line = []
        self._current_line.append(data)

    def _newline(self):
        if self._current_line is not None:
            self._current_paragraph.append(self._current_line)
            self._current_line = None

    def _close_open_paragraph(self):
        self._newline()
        if self._current_paragraph is not None:
            self._paragraphs.append(self._current_paragraph)
            self._current_paragraph = None

    def _open_paragraph(self):
        self._close_open_paragraph()
        self._current_paragraph = []

    def handle_data(self, data):
        self._append_chardata(data)

    def handle_starttag(self, tag, attrs):
        if tag == 'br':
            self._newline()
        elif tag == 'p':
            self._open_paragraph()

    def handle_endtag(self, tag):
        if tag == 'p':
            self._close_open_paragraph()

    def handle_startendtag(self, tag, attrs):
        self.handle_starttag(tag, attrs)

    def handle_entityref(self, name):
        self._append_chardata(unichr(name2codepoint[name]))

    def handle_charref(self, name):
        base = 10
        if name[:1] in "xX":
            name = name[1:]
            base = 16
        self._append_chardata(unichr(int(name, base)))

    def text(self):
        self._close_open_paragraph()
        return (u"\n".join('\n'.join(" ".join("".join(line).split())
                                     for line in para)
                           for para in self._paragraphs)
                + "\n")


def html_to_text(html):
    parser = _DeHTMLParser()
    parser.feed(html)
    parser.close()
    return parser.text()


class _HTMLShorten(HTMLParser):

    def __init__(self, max_len, shortcut):
        HTMLParser.__init__(self)
        self._max_len = max_len
        self._shortcut = shortcut
        self._text = []
        self._len = 0
        self._tag_list = []
        self._no_html = False

    def handle_starttag(self, tag, attrs):
        if tag != "p" and tag != "br":
            if self._len <= self._max_len - len(self._shortcut):
                self._tag_list.append(tag)
                self._text.append(self.get_starttag_text())

    def handle_endtag(self, tag):
        if self._len <= self._max_len and len(self._tag_list):
            end = self._tag_list.pop()
            if end == tag:
                self._text.append("</")
                self._text.append(tag)
                self._text.append(">")
            else:
                self._no_html = True


    def handle_startendtag(self, tag, attrs):
        pass

    def handle_data(self, data):
        if self._len <= self._max_len:
            data_len = len(data)
            if self._len + data_len <= self._max_len:
                self._text.append(data)
                self._len += data_len
            else:
                self._text.append(data[0:self._max_len - self._len -
                                   len(self._shortcut)])
                self._len += data_len

    def handle_entityref(self, name):
        if self._len <= self._max_len:
            self._text.append("&")
            self._text.append(name)
            self._text.append(";")
            self._len += 1

    def handle_charref(self, name):
        if self._len <= self._max_len:
            self._text.append("&#")
            self._text.append(name)
            self._text.append(";")
            self._len += 1

    def get_text(self):
        if self._no_html:
            return None
        text = []
        text.append("<p>")
        text += self._text
        while len(self._tag_list):
            text.append("</")
            text.append(self._tag_list.pop())
            text.append(">")
        if self._len > self._max_len:
            text.append(self._shortcut)
        text.append("</p>")
        return "".join(text)


def no_none(text):
    if text == 'None' or not text:
        return ''
    return text

def NA(text):
    if text == 'None' or not text:
        return '-/-'
    return text

def shorten(text, length=40):
    shortcut = "..."
    try:
        len(text)
        if text and len(text) > length:
            text = text[0:length-len(shortcut)] + shortcut
    except:
        log.warning('Trying to shorten a non string type: %s' % text)
    return text

def html_shorten(text, length=40):
    if text == "":
        return ""

    if text == None:
        log.warning('Trying to shorten None. Not string type.')
        return ""

    shortcut = "..."
    parser = _HTMLShorten(length, shortcut)
    parser.feed(text)
    parser.close()
    text_html = parser.get_text()
    if text_html:
        return literal(text_html)
    log.warning('Trying to shorten invalid HTML.')
    return shorten(html_to_text(text))

# vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8:
