# -*- coding: utf-8 -*-

import cgi
import re
import logging
import datetime

import formencode
from formencode import ForEach, All
from formencode.validators import Bool, Int, String, FieldsMatch, \
    StringBoolean, TimeConverter, FormValidator, MaxLength, \
    FileUploadKeeper as BaseFileUploadKeeper, OneOf, Wrapper, Regex, Invalid

from formed.meta.semantic import SemanticError, DateChecker
from formed.meta import UNKNOWN_DATE

from mpulsweb.lib.translation import _
from mpulsweb.lib.db import db
from mpulsweb.lib.helpers import format_date
from mpulsweb.lib.base import session

from mpulsweb.model.document import GlobalDocumentAPI, CaseDocumentAPI, \
    UserDocumentAPI, LogbookDocumentAPI
from mpulsweb.model.logbook import clean_html


log = logging.getLogger(__name__)


class ValidatorStateObject:

    pass


class BaseFormValidator(formencode.Schema):

    allow_extra_fields = True
    filter_extra_fields = False


class KeineAngabe(formencode.validators.FancyValidator):

    messages = {
        'invalid': u'"Keine Angabe" ist keine gültige Angabe.'
    }

    def _to_python(self, value, state):
        return value.strip()

    def validate_python(self, value, state):
        if value == '-1':
            raise formencode.Invalid(self.message("invalid", state),
                                     value, state)


class FileUploadKeeper(BaseFileUploadKeeper):

    # circumvent https://github.com/formencode/formencode/issues/71
    accept_iterator = True

class MultipleFileUpload(formencode.validators.FancyValidator):

    """Handle the upload of multiple files"""

    upload_key = 'upload'

    accept_iterator = True

    def _convert_to_python(self, value, state):
        upload = value.get(self.upload_key)
        if not isinstance(upload, list):
            upload = [upload]
        return [{'filename': entry.filename, 'content': entry.value}
                for entry in upload
                if isinstance(entry, cgi.FieldStorage)]


class FileNotEmptyCheck(formencode.validators.FancyValidator):

    """Checks if the filename-field of the provided file-dict is empty.
    This means that the user has not provided a file. Further check if
    the file is largger than the max allowed filesize."""

    max_size = 10000000  # 10MB = 10000000 bytes
    max_size_text = "10MB"

    messages = {
        'nofile': (u'Bitte geben Sie eine Datei zum Hochladen an.'),
        'toobig': ""  # set in the constructor
    }

    # circumvent https://github.com/formencode/formencode/issues/71
    accept_iterator = True

    def __init__(self, **kw):
        """This constructor allows the user to set the maximum upload size.
        The caller may specify the maximum size in bytes with the
        max_size keyword argument. If the max_size argument is given,
        the caller should also pass the textual representation of the
        size as max_size_text. The text can usually give a more human
        friendly value than a text automatically derived from the size
        in bytes.

        The defaults are given by the class attributes max_size and
        max_size_text and are used if the keyword arguments have been
        omitted or if their values are None.
        """
        if kw.get("max_size") is None:
            kw["max_size"] = self.max_size
        max_size_text = kw.pop("max_size_text", None)
        if max_size_text is None:
            max_size_text = self.max_size_text
        messages = {
            "toobig": (u"Die Datei {} ist größer als die maximal zulässige"
                       u" Größe von %s." % (max_size_text,)),
            }
        super(FileNotEmptyCheck, self).__init__(messages=messages, **kw)

    def _to_python(self, value, state=None):
        for entry in value if isinstance(value, list) else [value]:
            if not entry.get('filename'):
                raise formencode.Invalid(self.message("nofile", state),
                                         entry, state)
            if len(entry.get('content')) > self.max_size:
                raise formencode.Invalid(
                    self.message("toobig", state).format(
                        entry.get('filename')),
                    entry, state)

        return value


class FileExistsChecker(FormValidator):

    """Checks if a given file (or filename) already exists in the database.
    If so raise an exception if the user did not want to overwrite it.
    """

    api_constructor_fields = ()
    __unpackargs__ = ('*', 'api_constructor_fields')

    def __init__(self, document_api_class, *args, **kw):
        self.document_api_class = document_api_class
        super(FormValidator, self).__init__(*args, **kw)

    def validate_python(self, field_dict, state):
        # if user gives a userdefined name for the file take this to check if
        # the file is already existent. Else take filename from file
        name = field_dict["name"]
        document_id = field_dict.get("id")

        constructor_args = tuple(field_dict.get(key)
                                 for key in self.api_constructor_fields)
        api = self.document_api_class(*constructor_args)

        errors = {}
        for doc in api.used_names_db(name=name):
            if doc.id != document_id:
                errors['name'] = _("A file with that name already exists")

        if errors:
            error_list = errors.items()
            error_list.sort()
            error_message = '<br>\n'.join(['%s: %s' % (ename, value)
                                           for ename, value in error_list])
            raise formencode.Invalid(error_message, field_dict, state,
                                     error_dict=errors)


class DateOrder(FormValidator):

    """Tests that the given datetimes are in the corrent order.
    First given date must be smaller the second datePass the list of
    field names in as `field_names`.
    """

    field_names = None
    validate_partial_form = False  # do not run is a prior test failed
    start_time_field = None
    end_time_field = None
    __unpackargs__ = ("start_time_field", "end_time_field")


    def __init__(self, start_date_field, end_date_field, *args, **kw):
        super(FormValidator, self).__init__(*args, **kw)
        self.start_date_field = start_date_field
        self.end_date_field = end_date_field
        if ((self.start_time_field is not None)
            != (self.end_time_field is not None)):
            raise TypeError("DateOrder requires both or none of"
                            " start_time_field and end_time_field to be given"
                            " and not None")

    def conv_dates(self, dict):
        # Dates are already converted to datetime objects here. as we want
        # german representation in error messages convert the values back to
        # german format
        for key, value in dict.iteritems():
            if isinstance(value, datetime.date):
                value = format_date(value)
                dict[key] = value
        return dict

    def validate_python(self, field_dict, state):
        start_date = field_dict[self.start_date_field]
        end_date = field_dict[self.end_date_field]
        errors = {}
        if start_date is not None and end_date is not None:
            if end_date < start_date:
                errors[self.end_date_field] \
                    = _("The end date must not be before the start date")

            if start_date == end_date and self.start_time_field is not None:
                start_time = field_dict[self.start_time_field]
                end_time = field_dict[self.end_time_field]
                if (start_time is not None and end_time is not None
                    and end_time < start_time):
                    errors[self.end_time_field] \
                        = _("The end time must not be before the start time")

        if errors:
            error_list = errors.items()
            error_list.sort()
            error_message = '<br>\n'.join(['%s: %s' % (name, value)
                                           for name, value in error_list])
            raise formencode.Invalid(error_message, self.conv_dates(field_dict),
                                     state, error_dict=errors)


class FormedDateChecker(formencode.validators.FancyValidator):

    formed_checker = DateChecker()

    def __init__(self, *args, **kw):
        super(FormedDateChecker, self).__init__(*args, **kw)
        self._allow_unknown = kw.get("allow_unknown")

    def _to_python(self, value, state):
        try:
            return self.formed_checker.convert(value, None)
        except SemanticError, e:
            raise Invalid(e.value, value, state)

    def validate_python(self, value, state):
        if not self._allow_unknown and value == UNKNOWN_DATE:
            raise Invalid("Das Datum darf nicht 'Unbekannt' sein", value, state)
        try:
            return self.formed_checker.check(value, None)
        except SemanticError, e:
            raise Invalid(e.value, value, state)

    def _from_python(self, value, state):
        if value:
            return format_date(value)
        return value


class TimeDeltaCheck(FormValidator):

    """Tests that the given datetimes do not have a too large timedelta.
    """

    field_names = None
    validate_partial_form = False  # do not run is a prior test failed
    __unpackargs__ = ('*', 'field_names')
    days = None

    def __init__(self, *args, **kw):
        super(FormValidator, self).__init__(*args, **kw)
        if len(self.field_names) < 2:
            raise TypeError("FieldsMatch() requires at least two field names")

    def conv_dates(self, dict):
        # Dates are already converted to datetime objects here. as we want
        # german representation in error messages convert the values back to
        # german format
        for key, value in dict.iteritems():
            if isinstance(value, datetime.date):
                value = format_date(value)
                dict[key] = value
        return dict

    def validate_python(self, field_dict, state):
        begin = field_dict[self.field_names[0]]
        end = field_dict[self.field_names[1]]

        errors = {}
        if begin > end:
            begin, end = end, begin
        td = end - begin
        if self.days:
            if td.days > self.days:
                errors[self.field_names[0]] = _(u"The dates must be at most "
                                                "%s Days apart.") % self.days
                errors[self.field_names[1]] = _(u"The dates must be at most "
                                                "%s Days apart.") % self.days
        if errors:
            error_list = errors.items()
            error_list.sort()
            error_message = '<br>\n'.join(['%s: %s' % (name, value)
                                           for name, value in error_list])
            raise formencode.Invalid(error_message, self.conv_dates(field_dict),
                                     state, error_dict=errors)


class DurationCheck(formencode.validators.FancyValidator):

    messages = {
        'invalid_format': u'Bitte geben Sie die Dauer in Minuten als Zahl an.',
        'invalid_duration': u'Bitte geben Sie eine gültige Dauer an.',
    }

    def _to_python(self, value, state):
        try:
            duration = int(value)
        except:
            raise formencode.Invalid(self.message("invalid_format", state),
                                     value, state)
        try:
            return datetime.timedelta(minutes=duration)
        except:
            raise formencode.Invalid(self.message("invalid_duration", state),
                                     value, state)


class OldPassword(formencode.schema.FancyValidator):

    messages = {'oldpasswd': u'Sie müssen das alte Passwort korrekt eingeben.'}

    def validate_python(self, value, state):
        if not (value == session['USER_AUTHORIZED'].password):
            raise formencode.Invalid(self.message('oldpasswd', state),
                                     value, state)


class SecurePassword(formencode.validators.FancyValidator):

    min = 12
    non_letter = 2
    non_letter_literal = "zwei"
    letter_regex = re.compile(r'[a-zA-Z]')

    messages = {
        'too_few': (u'Ihr Passwort muss mindestens %(min)i'
                    u' Zeichen lang sein'),
        'non_letter': (u'Sie müssen mindestens %(non_letter)s'
                       u' Nicht-Buchstaben in Ihrem Passwort verwenden.'),
        'too_few_letter': (u'Das Passwort darf nicht nur aus Zahlen '
                           u'oder Sonderzeichen bestehen.'),
        'only_ascii': u'Das Passwort darf nur aus ASCII-Zeichen bestehen.'
    }

    def _to_python(self, value, state):
        # _to_python gets run before validate_python.  Here we
        # strip whitespace off the password, because leading and
        # trailing whitespace in a password is too elite.
        return value.strip()

    def validate_python(self, value, state):
        try:
            unicode(str(value), "ascii")
        except (UnicodeDecodeError, UnicodeEncodeError):
            raise formencode.Invalid(self.message("only_ascii", state),
                                     value, state)
        if len(value) < self.min:
            raise formencode.Invalid(self.message("too_few", state,
                                                  min=self.min),
                                     value, state)
        non_letters = self.letter_regex.sub('', value)
        if len(non_letters) < self.non_letter:
            raise formencode.Invalid(self.message(
                "non_letter", state, non_letter=self.non_letter_literal),
                value, state)
        if len(non_letters) == len(value):
            raise formencode.Invalid(self.message("too_few_letter", state),
                                     value, state)


class NewUserGroupCheck(formencode.validators.FancyValidator):

    """Validate names for new not-yet existing groups."""

    messages = {
        'group_exists': (u'Es existiert bereits eine Benutzergruppe mit'
                         u' dem Namen "%(name)s"!')
    }

    def _to_python(self, value, state):
        return value.strip()

    def validate_python(self, value, state):
        result = db.query("SELECT id FROM ka_benutzergruppe_tbl_view"
                          " WHERE name = %(name)s", {'name': value})
        if result is not None:
            raise formencode.Invalid(self.message("group_exists", state,
                                                  name=value),
                                     value, state)


class UserGroupExistsCheck(formencode.validators.FancyValidator):

    """Check editing a user group is OK.

    The user group's ID is given by the value for the key "id" in the
    form. There should also be a potentially new name for the group in
    the form under the key "name". Both values are OK if the group
    exists and either already has the given name or that name is not in
    use yet.
    """

    messages = {
        'group_exists': (u'Es existiert bereits eine Benutzergruppe mit'
                         u' dem Namen "%(name)s"!'),
        'unknown_group': (u"Die umzubenennende Gruppe existiert nicht oder"
                          u" nicht mehr!"),
    }

    field_names = None
    validate_partial_form = True
    __unpackargs__ = ('*', 'field_names')

    def __init__(self, *args, **kw):
        super(UserGroupExistsCheck, self).__init__(*args, **kw)
        if len(self.field_names) < 2:
            raise TypeError("UserGroupExistsCheck() requires at least two"
                            " field names")

    def validate_partial(self, value, state):
        group_id = value[self.field_names[0]]
        new_name = value[self.field_names[1]]

        group_exists, rename_ok = db.query(
            "SELECT"
            " EXISTS (SELECT * FROM ka_benutzergruppe_tbl_view"
            "         WHERE id = %(id)s)"
            " AS group_exists,"
            " NOT EXISTS (SELECT * FROM ka_benutzergruppe_tbl_view"
            "             WHERE id <> %(id)s AND name = %(name)s)"
            " AS rename_ok;",
            {'id': group_id, 'name': new_name})

        if not group_exists:
            raise formencode.Invalid(self.message("unknown_group", state),
                                     value, state)
        if not rename_ok:
            raise formencode.Invalid(
                "Cannot rename group", value, state,
                error_dict={"name": self.message("group_exists", state,
                                                 name=new_name)})


class LoginCheck(formencode.validators.FancyValidator):

    psql_max = 63
    login_max = 0
    valid_char = re.compile(r'^[a-z_0-9]+$')

    messages = {
        'too_long': (u'Die Anmeldekennung darf nicht länger als %(max)i '
                     u'Zeichen lang sein'),
        'invalid_char': (u'Die Anmeldekennung darf nur Kleinbuchstaben,'
                         u' Zahlen oder Unterstriche enthalten'),
        'login_exists': (u'Es existiert bereits ein Nutzer mit der'
                         u' Anmeldekennung "%(login)s".'),
        'invalid_username': (u'Die Anmeldekennung darf nicht "adm", "admin", '
                             u'oder "cm" lauten,')
    }

    def _to_python(self, value, state):
        # _to_python gets run before validate_python.  Here we
        # strip whitespace off the password, because leading and
        # trailing whitespace in a password is too elite.
        user = session['USER_AUTHORIZED']
        agency = user.getAgency()
        prefix_str = "ka_%s_" % agency
        prefix_length = len(prefix_str)
        self.login_max = self.psql_max - prefix_length
        return value.strip()

    def validate_python(self, value, state):
        if len(value) > (self.login_max):
            raise formencode.Invalid(self.message("too_long", state,
                                                  max=self.login_max),
                                     value, state)
        if not self.valid_char.match(value):
            raise formencode.Invalid(self.message("invalid_char", state),
                                     value, state)
        if value in ['adm', 'admin', 'cm']:
            raise formencode.Invalid(self.message("invalid_username", state),
                                     value, state)
        try:
            conn, cur = None, None
            loginname = "_".join(["ka", state.agency, value])
            fields = {'login': loginname}
            try:
                conn = db.getConnection()
                cur = conn.cursor()
                cur.execute("SELECT id FROM ka_benutzer_tbl_view"
                            " WHERE login = %(login)s", fields)
                result = cur.fetchone()
                if result:
                    raise formencode.Invalid(self.message("login_exists",
                                                          state, login=value),
                                             value, state)
            finally:
                db.recycleConnection(conn, cur)
        except StandardError:
            log.exception("Fehler beim Validieren der Anmeldekennung.")


class MaxMinInt(formencode.validators.FancyValidator):

    """
    Convert a value to an integer.

    Example::

        >>> Int.to_python('10')
        10
        >>> Int.to_python('ten')
        Traceback (most recent call last):
        Invalid: Please enter an integer value
    """

    messages = {
        'integer': u"Bitte geben Sie einen Ganzzahlenwert ein.",
        'tooLow': (u"Bitte geben Sie einen Wert größer oder gleich"
                   u" %(min)i ein."),
        'tooHigh': (u"Bitte geben Sie einen Wert kleiner oder gleich"
                    u" %(max)i ein.")
    }

    min = None
    max = None

    def __initargs__(self, args):
        if not self.min is None:
            self.min = int(self.min)
        if not self.max is None:
            self.max = int(self.max)

    def _to_python(self, value, state):
        try:
            return int(value)
        except (ValueError, TypeError):
            raise formencode.Invalid(self.message('integer', state),
                                     value, state)

    def validate_python(self, value, state):
        if not self.min is None and value < self.min:
            msg = self.message("tooLow", state, min=self.min)
            raise formencode.Invalid(msg, value, state)
        if not self.max is None and value > self.max:
            msg = self.message("tooHigh", state, max=self.max)
            raise formencode.Invalid(msg, value, state)

    _from_python = _to_python


class UploadGlobalDocumentForm(BaseFormValidator):

    def __init__(self, max_size=None, max_size_text=None):
        """Instantiate the upload validator.
        The maximum file size can be set with the arguments max_size
        and max_size_text which are passed to FileNotEmptyCheck as
        keyword arguments. See that class for the meaning of the
        parameters.
        """
        super(UploadGlobalDocumentForm, self).__init__(
            file = All(FileNotEmptyCheck(not_empty=True,
                                         max_size=max_size,
                                         max_size_text=max_size_text),
                       MultipleFileUpload()))

    pre_validators = [formencode.variabledecode.NestedVariables()]



class RenameGlobalDocumentForm(BaseFormValidator):

    name = String(not_empty=True)
    id = Int(if_missing=None)
    chained_validators = [FileExistsChecker(GlobalDocumentAPI)]


class RenameCaseDocumentForm(BaseFormValidator):

    name = String(not_empty=True)
    id = Int(if_missing=None)
    case = Int(if_missing=None)
    chained_validators = [FileExistsChecker(CaseDocumentAPI, 'case')]


class RenameUserDocumentForm(BaseFormValidator):

    name = String(not_empty=True)
    id = Int(if_missing=None)
    case = Int(if_missing=None)
    chained_validators = [FileExistsChecker(UserDocumentAPI)]


class RenameLogbookDocumentForm(BaseFormValidator):

    name = String(not_empty=True)
    id = Int(if_missing=None)
    logbook_id = Int(if_missing=None)
    chained_validators = [FileExistsChecker(LogbookDocumentAPI, 'logbook_id')]


class UploadCaseDocumentForm(BaseFormValidator):

    def __init__(self, max_size=None, max_size_text=None):
        """Instantiate the upload validator.
        The maximum file size can be set with the arguments max_size
        and max_size_text which are passed to FileNotEmptyCheck as
        keyword arguments. See that class for the meaning of the
        parameters.
        """
        super(UploadCaseDocumentForm, self).__init__(
              file = All(FileNotEmptyCheck(not_empty=True,
                                           max_size=max_size,
                                           max_size_text=max_size_text),
                         MultipleFileUpload()))
    pre_validators = [formencode.variabledecode.NestedVariables()]

    case = Int(not_empty=True)


class LogbookCaseDocumentForm(BaseFormValidator):

    def __init__(self, max_size=None, max_size_text=None):
        """Instantiate the upload validator.
        The maximum file size can be set with the arguments max_size
        and max_size_text which are passed to FileNotEmptyCheck as
        keyword arguments. See that class for the meaning of the
        parameters.
        """
        super(LogbookCaseDocumentForm, self).__init__(
              file = All(FileNotEmptyCheck(not_empty=True,
                                           max_size=max_size,
                                           max_size_text=max_size_text),
                         MultipleFileUpload()))
    pre_validators = [formencode.variabledecode.NestedVariables()]

    logbook = Int(not_empty=True)


class EditSettingsForm(BaseFormValidator):

    # We do not allow any fields that are not explicitly listed to make
    # sure that users cannot try to modify settings stored in
    # ka_konfiguration_tbl that are not listed here.
    allow_extra_fields=False
    filter_extra_fields=True

    ka_name = String()
    ka_adresse = String()

    anon_transfer = String(if_missing='off')


class NewPasswordForm(BaseFormValidator):

    passwd = SecurePassword()
    passwd2 = String(min=1)
    chained_validators = [FieldsMatch('passwd', 'passwd2')]


class CheckedNewPasswordForm(NewPasswordForm):

    oldpasswd = OldPassword(not_empty=True)


class NewUserForm(BaseFormValidator):

    def __init__(self):
        formencode.api.set_stdtranslation(
            domain="FormEncode", languages=["de"])

    firstname = String(not_empty=True)
    lastname = String(not_empty=True)
    login = All(String(not_empty=True), LoginCheck())
    activated = StringBoolean(if_missing=False)
    newpass = StringBoolean(if_missing=False)
    agencys = ForEach(Int(), convert_to_list=True)


def _convert_to_timedelta(value):
    if value is None:
        return None
    return datetime.timedelta(value)


class CreateAppointmentForm(BaseFormValidator):

    start_date = FormedDateChecker(not_empty=True)
    start_time = TimeConverter(if_empty=datetime.time(0, 0), use_datetime=True)

    end_date = FormedDateChecker()
    end_time = TimeConverter(use_datetime=True)

    chained_validators = [DateOrder('start_date', 'end_date',
                                    'start_time', 'end_time')]
    title = All(String(not_empty=True), MaxLength(128))
    trigger = All(Wrapper(to_python=_convert_to_timedelta,
                          from_python=lambda td: td.days,
                          if_missing=None), MaxMinInt(max=3650, min=0))

    def to_python(self, *args, **kw):
        try:
            return super(CreateAppointmentForm, self).to_python(*args, **kw)
        except formencode.Invalid, error:
            # depending on what was wrong with the input, the trigger
            # value may have been converted to a timedelta or not. If it
            # has been converted already, convert it back to string
            # explicitly, so that it is rendered correctly in the form.
            trigger = error.value["trigger"]
            if isinstance(trigger, datetime.timedelta):
                error.value["trigger"] = trigger.days
            raise


class EditUserForm(NewUserForm):

    login = None


class NewUserGroupForm(BaseFormValidator):

    name = NewUserGroupCheck(not_empty=True)


class EditUserGroupForm(BaseFormValidator):

    id = Int(not_empty=True)
    name = String(not_empty=True)
    standin = ForEach(Int(), convert_to_list=True)

    chained_validators = [UserGroupExistsCheck("id", "name")]


class HtmlCleanedString(String):

    def to_python(self, *args, **kwargs):
        string = super(HtmlCleanedString, self).to_python(*args, **kwargs)
        return clean_html(string)


class EditLogbookForm(BaseFormValidator):

    # Note: art and typ are actually dynamic in that they're actually
    # defined by the JSON configuration file (logbook/types). In the
    # lobook model "art" and "typ" are practically hard-wired to integer
    # fields, though, so it shouldn't be too bad to hard-wire their
    # types here, too.

    art = Int(not_empty=True)
    date = FormedDateChecker(not_empty=True)
    time = TimeConverter(not_empty=True, use_datetime=True)
    duration = DurationCheck()
    short_notice = MaxLength(80, if_missing="")
    notice = HtmlCleanedString()
    typ = Int(if_missing=None)


class CreateLogbookForm(EditLogbookForm):

    def __init__(self, max_size=None, max_size_text=None):
        """Instantiate the upload validator.
        The maximum file size can be set with the arguments max_size
        and max_size_text which are passed to FileNotEmptyCheck as
        keyword arguments. See that class for the meaning of the
        parameters.
        """
        super(CreateLogbookForm, self).__init__(
              file = All(FileNotEmptyCheck(max_size=max_size,
                                           max_size_text=max_size_text),
                         MultipleFileUpload()))
    pre_validators = [formencode.variabledecode.NestedVariables()]


class RenameSavedSearchForm(BaseFormValidator):

    name = MaxLength(128)


class SetEditorForm(BaseFormValidator):

    pass


class SetStandinForm(BaseFormValidator):

    filter_extra_fields = True
    case_id = Int()
    standin = ForEach(Int(), convert_to_list=True)


class EvaluationFormValidator(BaseFormValidator):

    def __init__(self, *args, **kw):
        super(EvaluationFormValidator, self).__init__(*args, **kw)
        # Add the validator for mpuls-toggle-all-typelist here because
        # of the unusual name. The prefix mpuls-toggle-all was chosen
        # for the toggle all checkboxes feature to avoid collisions with
        # existing form fields.
        self.add_field("mpuls-toggle-all-typelist", Bool(default=True))

    id = String(if_missing='0')

    start_date = FormedDateChecker(not_empty=True, if_missing='')
    end_date = FormedDateChecker(not_empty=True, if_missing='')

    # FIXME: typelist and phase are actually lists of ints, not lists of
    # strings. Unfortunately, we cannot just convert the list items to
    # ints because there is code that expects them to be
    # strings. However, the least we can and should do is verify that
    # the values look like int literals.
    typelist = ForEach(Regex("^-?[0-9]+$"), convert_to_list=True)
    phase = ForEach(Regex("^-?[0-9]+$"), convert_to_list=True)

    type_ending = ForEach(Int(), convert_to_list=True)
    show_percent = Bool(if_missing=False)
    selected_ids = Wrapper(to_python=lambda s: [int(item)
                                                for item in s.split()],
                           from_python=lambda ids: " ".join(map(unicode, ids)),
                           if_missing=())
    chained_validators = [DateOrder('start_date', 'end_date')]


class SearchCaseForm(BaseFormValidator):

    state = ForEach(Int(), convert_to_list=True)
    own = Int(if_missing=0)
    standin = Int(if_missing=0)
    editor = Int(if_missing=-1)
    phase = ForEach(Int(), convert_to_list=True)
    sdate = FormedDateChecker(if_missing=None)
    edate = FormedDateChecker(if_missing=None)
    gender = Int(if_missing=-2)

    chained_validators = [DateOrder('sdate', 'edate')]


class EvaluationSearchCaseForm(SearchCaseForm):

    federal_state = ForEach(Int(), convert_to_list=True)


class BundleActionForm(BaseFormValidator):

    filter_extra_fields = True

    all_cases = Int(if_missing=0)
    action = All(String(not_empty=True),
                 OneOf(['no_action',
                        'markdelete', 'delete',
                        'markanonymize', 'anonymize',
                        'restore',
                        'standin', 'transfer',
                        'exportXML', 'exportCSV', 'exportXLS', 'exportEXLS',
                        'formletter:1', 'formletter:2', 'formletter:3',
                        'formletter:4', 'formletter:5',
                        'evaluate']))
    case_id = ForEach(Int(), convert_to_list=True)
    all_ids = Wrapper(to_python=lambda s: [int(item) for item in s.split()])


class ExportSelectorValidator(BaseFormValidator):

    filter_extra_fields = True
    exportanonym = Bool()
    exporttype = String()
    exportselection = ForEach(String(), convert_to_list=True)
    squash_lines = Bool()


class SetBundleEditorForm(BaseFormValidator):

    editor = Int(not_empty=True)


class SetBundleStandinForm(BaseFormValidator):

    standin = ForEach(Int(), convert_to_list=True)


class DeleteUserHelperForm(BaseFormValidator):

    filter_extra_fields = True

    editor = Int()
    user_id = Int()
    case_id = ForEach(Int())


class NewCaseForm(BaseFormValidator):

    statement_signed = Bool(if_missing=False)


# vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8
