-- Upgrade script for the template database for more flexible roles
-- See mpuls/issue2530
--
-- This script has to be applied to the template database as well as the
-- normal application databases. The latter is done as usual with
-- mpulsdbadm.py upgrade_db. The former can be done with the following
-- commands as linux user postgres:
--
-- psql -c "UPDATE pg_database SET datallowconn = TRUE where datname='mpuls_template';"
-- psql -d mpuls_template -f sql/upgrades/1.2.6/01_more_roles.sql
-- psql -c "UPDATE pg_database SET datallowconn = FALSE where datname='mpuls_template';"


SET ROLE postgres;

CREATE OR REPLACE FUNCTION mpuls_create_role (
        username varchar(64),
        grouprole varchar(64), 
        agency varchar(52)
  ) RETURNS RECORD LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER
  AS $$
  DECLARE 
    db_name  varchar(64);
    db_user  varchar(128);
    db_group varchar(64);
    pw       varchar(13);
    result   RECORD;
  BEGIN
    -- IF position(quote_literal(agency) in session_user) != 4
    IF position(agency in session_user) != 4
    THEN
      RAISE EXCEPTION 
          'Permission denied to create role %s in agency % by %.',
          quote_literal(username), quote_literal(agency), session_user;
    ELSE
      db_name := quote_ident('ka_' || agency || '_db');
      db_user := quote_ident('ka_' || agency || '_' || username);
      db_group = quote_ident(mpuls_role_group_name(grouprole, agency));
      pw := mpuls_passgen(12);
      EXECUTE 'CREATE ROLE ' || db_user 
          || ' LOGIN NOSUPERUSER INHERIT NOCREATEDB NOCREATEROLE'
          || ' PASSWORD ' || quote_literal(pw);
      EXECUTE 'GRANT ' || db_group || ' TO ' || db_user;
      SELECT INTO result db_name::varchar(64), db_user::varchar(64), pw::varchar(13);
      RETURN result;
    END IF;

  END;
$$;
ALTER FUNCTION mpuls_create_role(varchar, varchar, varchar)
    OWNER TO mpuls_role_adm;
REVOKE ALL ON FUNCTION mpuls_create_role(varchar, varchar, varchar)
    FROM public;
GRANT EXECUTE ON FUNCTION mpuls_create_role(varchar, varchar, varchar)
    TO admin;

CREATE OR REPLACE FUNCTION mpuls_role_group_name (
        grouprole varchar(64), 
        agency varchar(52)
  ) RETURNS varchar(64) LANGUAGE 'plpgsql' IMMUTABLE
AS $$
  BEGIN
    RETURN mpuls_role_group_name_default(grouprole, agency);
  END;
$$;
ALTER FUNCTION mpuls_role_group_name(varchar, varchar)
    OWNER TO mpuls_role_adm;
REVOKE ALL ON FUNCTION mpuls_role_group_name(varchar, varchar)
    FROM public;
GRANT EXECUTE ON FUNCTION mpuls_role_group_name(varchar, varchar)
    TO admin;


-- Default implementation of mpuls_role_group_name. The only allowed
-- groups are 'admin' and 'cm'. All other groups are invalid.
CREATE OR REPLACE FUNCTION mpuls_role_group_name_default (
        grouprole varchar(64), 
        agency varchar(52)
  ) RETURNS varchar(64) LANGUAGE 'plpgsql' IMMUTABLE
AS $$
  DECLARE
    normalized_grouprole VARCHAR(64);
  BEGIN
    normalized_grouprole := lower(grouprole);
    IF normalized_grouprole in ('admin', 'cm') THEN
      RETURN 'ka_' || agency || '_' || normalized_grouprole;
    ELSE
      RAISE EXCEPTION 'Invalid group name %.', quote_literal(grouprole);
    END IF;
  END;
$$;
ALTER FUNCTION mpuls_role_group_name_default(varchar, varchar)
    OWNER TO mpuls_role_adm;
REVOKE ALL ON FUNCTION mpuls_role_group_name_default(varchar, varchar)
    FROM public;
GRANT EXECUTE ON FUNCTION mpuls_role_group_name_default(varchar, varchar)
    TO admin;
