from datetime import date

import xml.etree.ElementTree as ET

from formed.meta.data import VirtualLeaf

LIBFORMED_XML_CONTAINER_VERSION = "6"

XML_ENCODERS = {
    unicode: lambda s: s,
    date: lambda s: "%04d-%02d-%02d" % (s.year, s.month, s.day)
}


def serialize_case(instance, external=None):
    """Return the structure instance tree as an XML string.
    If the parameter external is given and not None, it will be treated
    as a callable which is called with the ElementTree.Element instance
    containing the XML representation of the case.  The callable may
    modify the element instance to add more information, like the
    logbook entries associated with the case.
    """
    formed = instance.meta_structure.meta_tree
    cases = create_cases_container(formed.getVersion())
    case = add_case(cases, instance)
    if external is not None:
        external(case)
    xml = ET.ElementTree(cases)
    return ET.tostring(xml.getroot())


def create_cases_container(formed_type):
    return ET.Element("cases",
                      {"version": LIBFORMED_XML_CONTAINER_VERSION,
                       "formed-type": formed_type})


def add_case(cases, instance):
    """Adds the serialized simplified instance to the cases element"""
    case = ET.SubElement(cases, "case")
    add_node(case, instance.root)
    return case


def add_node(parent, node):
    meta_node = node.meta_node
    element_name = meta_node.name
    if element_name is None:
        element_name = "master"
    element = ET.SubElement(parent, element_name)
    add_leaf(element, "uuid_id", node.uuid)
    for nc in meta_node.ordered_leaves:
        if isinstance(nc, VirtualLeaf):
            # ignore VirtualLeafs
            continue
        name = nc.getName()
        add_leaf(element, name, node.get((name,)))
    for meta_rg in meta_node.ordered_repeat_groups:
        for rg_instance in node.get_repeat_groups(meta_rg.name).itervalues():
            add_node(element, rg_instance)

def add_leaf(element, name, value):
    """Add a sub-element to element with value as content if value is not None.
    This function creates a new element as a sub-element of the element
    given as the parameter element. The parameter name is the name of
    the new sub-element. The text-content of the new element is the
    encoded form of the value. This encoding is done with the functions
    in XML_ENCODERS. Therefore the type of value must be one of the
    types used as keys in XML_ENCODERS.
    If the value is None, no sub-element is created and element is not
    modified.
    """
    if value is not None:
        leaf_element = ET.SubElement(element, name)
        leaf_element.text = XML_ENCODERS.get(value.__class__, unicode)(value)
