#!/bin/sh

set -e

usage()
{
cat << EOF
usage: $0 options source

This script builds a release tarball from given source directory.

OPTIONS:
   -h       Show this message
   -n       Name of the release (Defaults to name of archived source-revision)
   -o       Builddir (Default /tmp)
   -r       Revision number used to build the release
            (Default: tip of working directory)
   -c       Build release as tar.gz file (Default: False)
EOF
}

# DEFAULTS
VERBOSE=0
NAME=
BUILDDIR=/tmp
REVISION=
COMPRESS=0

# Parse Options
while getopts "hcn:o:r:" OPTION; do
    case $OPTION in
        h)
            usage
            exit 1
            ;;
        n)
            NAME=$OPTARG
            ;;
        v)
            VERBOSE=1
            ;;
        o)
            BUILDDIR=$OPTARG
            ;;
        r)
            REVISION=$OPTARG
            ;;
        c)
            COMPRESS=1
            ;;
        ?)
            # Will be set if the optstring begins with a ":". In this case
            # all options not defined in the optstring will OPTION set to
            # "?". Otherwise the shell reports an error on the unknown
            # option.
            usage
            exit
            ;;
    esac
done

# Forget about options. arguments now begin a position 1
shift $(($OPTIND - 1))
SOURCE=${1}

# Check Arguments
if [[ -z $SOURCE ]]; then
    echo "Error! Argument required!";
    usage;
    exit 1;
fi

# Check if source dir exists
if [ ! -d $SOURCE ]; then
    echo "Error! Can not find source dir!";
    usage;
    exit 1;
else
    DIR=`readlink -f ${SOURCE}`
fi

if [ -z ${REVISION} ]; then
    # FIXME: Using the tip revision is probably not the right thing to
    # do! tip is the most recent revision in the repository, regardless
    # of branches.
    REVISION=$(hg log -r tip --template="{rev}" -R $SOURCE)
fi

# Build releasename
if [ -z $NAME ]; then
    echo "Error! Please provide name of the distribution!";
    exit 1;
fi
RELEASENAME=${NAME}-${REVISION}

ARCHIVEDIR=${BUILDDIR}/${RELEASENAME}

# Archive
hg archive -r ${REVISION} -S -t files -R $SOURCE $ARCHIVEDIR


# For convenience, switch to the archive directory for the rest of the
# script
cd $ARCHIVEDIR

# Compile translations. To avoid hard-coding a list of subrepositories
# here, we simply assume that all *.po files in the archive are
# translation files and need to be converted to *.mo files. Using
# heuristics to determine which subdirectories contain setup.py files
# that should be called for translations doesn't suffice, because e.g.
# mpuls-S-Bewerbungserver has a subdirectory translations/ that contains
# translations but doesn't have setup.py file and it doesn't cover
# subdirectories where the setup.py mechanism doesn't suffice because it
# doesn't support multiple catalogs.
for f in $(find . -name \*.po); do
    msgfmt ${f} -o $(echo ${f} | sed -e "s/\.po$/.mo/g");
done


if test -d base; then
    (
        cd base
        DATE=$(date +%d.%m.%Y)
        sed -i -e s/DISTRIBUTION/${NAME}/g mpulsweb/lib/config.py
        sed -i -e s/REVISION/${REVISION}/g mpulsweb/lib/config.py
        sed -i -e s/RELEASEDATE/${DATE}/g mpulsweb/lib/config.py
    )
elif test -d application/mpulsweblight; then
    sed -i -e "s/^__version_info__.*/__version_info__ = '${REVISION}'.split('.')/g" \
	application/mpulsweblight/__init__.py
fi

# Compress release
if [ ${COMPRESS} -eq 1 ]; then
    echo "Compressing archive"
    ARCHIVE=${BUILDDIR}/${RELEASENAME}.tar.gz
    tar cz -C ${BUILDDIR} --owner=0 --group=0 -f $ARCHIVE ${RELEASENAME}
else
    ARCHIVE=$ARCHIVEDIR
fi

echo "Archive ready: ${ARCHIVE}"
